@@ -1,16 +1,18 @@
+/* $Cambridge: exim/exim-src/src/auths/cram_md5.h,v 1.2 2005/01/04 10:00:43 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
 
 typedef struct {
-  char *server_secret;
-  char *client_secret;
-  char *client_name;
+  uschar *server_secret;
+  uschar *client_secret;
+  uschar *client_name;
 } auth_cram_md5_options_block;
 
 /* Data for reading the private options. */
@@ -25,8 +27,8 @@ extern auth_cram_md5_options_block auth_cram_md5_option_defaults;
 /* The entry points for the mechanism */
 
 extern void auth_cram_md5_init(auth_instance *);
-extern int auth_cram_md5_server(auth_instance *, char *);
+extern int auth_cram_md5_server(auth_instance *, uschar *);
 extern int auth_cram_md5_client(auth_instance *, smtp_inblock *,
-                                smtp_outblock *, int, char *, int);
+                                smtp_outblock *, int, uschar *, int);
 
 /* End of cram_md5.h */
