@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/sieve.c,v 1.7 2005/02/17 11:58:26 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/sieve.c,v 1.17 2005/11/28 09:47:20 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) Michael Haardt 2003,2004 */
+/* Copyright (c) Michael Haardt 2003-2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This code was contributed by Michael Haardt. */
@@ -28,11 +28,16 @@
 /* Undefine it for UNIX-style \n end-of-line terminators (default). */
 #undef RFC_EOL
 
-/* Define this for development of the subaddress Sieve extension.   */
-/* The code is currently broken.                                    */
-#undef SUBADDRESS
+/* Define this for development of the Sieve extension "envelope-auth". */
+#undef ENVELOPE_AUTH
 
-/* Define this for the vacation Sieve extension.                    */
+/* Define this for development of the Sieve extension "notify".     */
+#undef NOTIFY
+
+/* Define this for the Sieve extension "subaddress".                */
+#define SUBADDRESS
+
+/* Define this for the Sieve extension "vacation".                  */
 #define VACATION
 
 /* Must be >= 1                                                     */
@@ -53,6 +58,12 @@ struct Sieve
   int keep;
   int require_envelope;
   int require_fileinto;
+#ifdef ENVELOPE_AUTH
+  int require_envelope_auth;
+#endif
+#ifdef NOTIFY
+  int require_notify;
+#endif
 #ifdef SUBADDRESS
   int require_subaddress;
 #endif
@@ -61,11 +72,13 @@ struct Sieve
   int vacation_ran;
 #endif
   uschar *vacation_directory;
+  const uschar *subaddress;
+  const uschar *useraddress;
   int require_copy;
   int require_iascii_numeric;
   };
 
-enum Comparator { COMP_OCTET, COMP_ASCII_CASEMAP, COMP_ASCII_NUMERIC };
+enum Comparator { COMP_OCTET, COMP_EN_ASCII_CASEMAP, COMP_ASCII_NUMERIC };
 enum MatchType { MATCH_IS, MATCH_CONTAINS, MATCH_MATCHES };
 #ifdef SUBADDRESS
 enum AddressPart { ADDRPART_USER, ADDRPART_DETAIL, ADDRPART_LOCALPART, ADDRPART_DOMAIN, ADDRPART_ALL };
@@ -91,6 +104,8 @@ static uschar str_cc_c[]="Cc";
 static const struct String str_cc={ str_cc_c, 2 };
 static uschar str_bcc_c[]="Bcc";
 static const struct String str_bcc={ str_bcc_c, 3 };
+static uschar str_auth_c[]="auth";
+static const struct String str_auth={ str_auth_c, 4 };
 static uschar str_sender_c[]="Sender";
 static const struct String str_sender={ str_sender_c, 6 };
 static uschar str_resent_from_c[]="Resent-From";
@@ -101,6 +116,14 @@ static uschar str_fileinto_c[]="fileinto";
 static const struct String str_fileinto={ str_fileinto_c, 8 };
 static uschar str_envelope_c[]="envelope";
 static const struct String str_envelope={ str_envelope_c, 8 };
+#ifdef ENVELOPE_AUTH
+static uschar str_envelope_auth_c[]="envelope-auth";
+static const struct String str_envelope_auth={ str_envelope_auth_c, 13 };
+#endif
+#ifdef NOTIFY
+static uschar str_notify_c[]="notify";
+static const struct String str_notify={ str_notify_c, 6 };
+#endif
 #ifdef SUBADDRESS
 static uschar str_subaddress_c[]="subaddress";
 static const struct String str_subaddress={ str_subaddress_c, 10 };
@@ -115,12 +138,16 @@ static uschar str_copy_c[]="copy";
 static const struct String str_copy={ str_copy_c, 4 };
 static uschar str_iascii_casemap_c[]="i;ascii-casemap";
 static const struct String str_iascii_casemap={ str_iascii_casemap_c, 15 };
+static uschar str_enascii_casemap_c[]="en;ascii-casemap";
+static const struct String str_enascii_casemap={ str_enascii_casemap_c, 16 };
 static uschar str_ioctet_c[]="i;octet";
 static const struct String str_ioctet={ str_ioctet_c, 7 };
 static uschar str_iascii_numeric_c[]="i;ascii-numeric";
 static const struct String str_iascii_numeric={ str_iascii_numeric_c, 15 };
 static uschar str_comparator_iascii_casemap_c[]="comparator-i;ascii-casemap";
 static const struct String str_comparator_iascii_casemap={ str_comparator_iascii_casemap_c, 26 };
+static uschar str_comparator_enascii_casemap_c[]="comparator-en;ascii-casemap";
+static const struct String str_comparator_enascii_casemap={ str_comparator_enascii_casemap_c, 27 };
 static uschar str_comparator_ioctet_c[]="comparator-i;octet";
 static const struct String str_comparator_ioctet={ str_comparator_ioctet_c, 18 };
 static uschar str_comparator_iascii_numeric_c[]="comparator-i;ascii-numeric";
@@ -306,8 +333,7 @@ while (nl>0 && hl>0)
   if (hc&0x80) return 0;
 #endif
   /* tolower depends on the locale and only ASCII case must be insensitive */
-  if ((nc&0x80) || (hc&0x80)) { if (nc!=hc) return 0; }
-  else if ((nc>='A' && nc<='Z' ? nc|0x20 : nc) != (hc>='A' && hc<='Z' ? hc|0x20 : hc)) return 0;
+  if ((nc>='A' && nc<='Z' ? nc|0x20 : nc) != (hc>='A' && hc<='Z' ? hc|0x20 : hc)) return 0;
   ++n;
   ++h;
   --nl;
@@ -318,7 +344,7 @@ return (match_prefix ? nl==0 : nl==0 && hl==0);
 
 
 /*************************************************
-*        Octet-wise glob pattern search          *
+*              Glob pattern search               *
 *************************************************/
 
 /*
@@ -328,231 +354,99 @@ Arguments:
 
 Returns:      0               needle not found in haystack
               1               needle found
+              -1              pattern error
 */
 
-static int eq_octetglob(const struct String *needle,
-  const struct String *haystack)
+static int eq_glob(const struct String *needle,
+  const struct String *haystack, int ascii_caseless)
 {
-struct String n,h;
+const uschar *n,*h,*nend,*hend;
+int may_advance=0;
 
-n=*needle;
-h=*haystack;
-while (n.length)
+n=needle->character;
+h=haystack->character;
+nend=n+needle->length;
+hend=h+haystack->length;
+while (n<nend)
   {
-  switch (n.character[0])
+  if (*n=='*')
     {
-    case '*':
-      {
-      int currentLength;
-
-      ++n.character;
-      --n.length;
-      /* The greedy match is not yet well tested.  Some day we may */
-      /* need to refer to the matched parts, so the code is already */
-      /* prepared for that. */
-#if 1
-      /* greedy match */
-      currentLength=h.length;
-      h.character+=h.length;
-      h.length=0;
-      while (h.length<=currentLength)
-        {
-        if (eq_octetglob(&n,&h)) return 1;
-        else /* go back one octet */
-          {
-          --h.character;
-          ++h.length;
-          }
-        }
-      return 0;
-#else
-      /* minimal match */
-      while (h.length)
-        {
-        if (eq_octetglob(&n,&h)) return 1;
-        else /* advance one octet */
-          {
-          ++h.character;
-          --h.length;
-          }
-        }
-      break;
-#endif
-      }
-    case '?':
-      {
-      if (h.length)
-        {
-        ++h.character;
-        --h.length;
-        ++n.character;
-        --n.length;
-        }
-      else return 0;
-      break;
-      }
-    case '\\':
-      {
-      ++n.character;
-      --n.length;
-      /* FALLTHROUGH */
-      }
-    default:
-      {
-      if
-        (
-        h.length==0 ||
-#if !HAVE_ICONV
-        (h.character[0]&0x80) || (n.character[0]&0x80) ||
-#endif
-        h.character[0]!=n.character[0]
-        ) return 0;
-      else
-        {
-        ++h.character;
-        --h.length;
-        ++n.character;
-        --n.length;
-        };
-      }
+    ++n;
+    may_advance=1;
     }
-  }
-return (h.length==0);
-}
-
-
-/*************************************************
-*   ASCII case-insensitive glob pattern search   *
-*************************************************/
-
-/*
-Arguments:
-  needle      UTF-8 pattern to search ...
-  haystack    ... inside the haystack
-
-Returns:      0               needle not found in haystack
-              1               needle found
-*/
-
-static int eq_asciicaseglob(const struct String *needle,
-  const struct String *haystack)
-{
-struct String n,h;
-
-n=*needle;
-h=*haystack;
-while (n.length)
-  {
-  switch (n.character[0])
+  else
     {
-    case '*':
-      {
-      int currentLength;
+    const uschar *npart,*hpart;
 
-      ++n.character;
-      --n.length;
-      /* The greedy match is not yet well tested.  Some day we may */
-      /* need to refer to the matched parts, so the code is already */
-      /* prepared for that. */
-#if 1
-      /* greedy match */
-      currentLength=h.length;
-      h.character+=h.length;
-      h.length=0;
-      while (h.length<=currentLength)
-        {
-        if (eq_asciicaseglob(&n,&h)) return 1;
-        else /* go back one UTF-8 character */
-          {
-          if (h.length==currentLength) return 0;
-          --h.character;
-          ++h.length;
-          if (h.character[0]&0x80)
-            {
-            while (h.length<currentLength && (*(h.character-1)&0x80))
-              {
-              --h.character;
-              ++h.length;
-              }
-            }
-          }
-        }
-      /* NOTREACHED */
-#else
-      while (h.length)
-        {
-        if (eq_asciicaseglob(&n,&h)) return 1;
-        else /* advance one UTF-8 character */
-          {
-          if (h.character[0]&0x80)
-            {
-            while (h.length && (h.character[0]&0x80))
-              {
-              ++h.character;
-              --h.length;
-              }
-            }
-          else
-            {
-            ++h.character;
-            --h.length;
-            }
-          }
-        }
-      break;
-#endif
-      }
-    case '?':
+    /* Try to match a non-star part of the needle at the current */
+    /* position in the haystack.                                 */
+    match_part:
+    npart=n;
+    hpart=h;
+    while (npart<nend && *npart!='*') switch (*npart)
       {
-      if (h.length)
+      case '?':
         {
-        ++n.character;
-        --n.length;
-        /* advance one UTF-8 character */
-        if (h.character[0]&0x80)
+        if (hpart==hend) return 0;
+        /* watch out: Do not match one character, but one UTF8 encoded character */
+        if ((*hpart&0xc0)==0xc0)
           {
-          while (h.length && (h.character[0]&0x80))
+          ++hpart;
+          while (hpart<hend && ((*hpart&0xc0)==0x80)) ++hpart;
+          }
+        else
+         ++hpart;
+        ++npart;
+        break;
+        }
+      case '\\':
+        {
+        ++npart;
+        if (npart==nend) return -1;
+        /* FALLTHROUGH */
+        }
+      default:
+        {
+        if (hpart==hend) return 0;
+        /* tolower depends on the locale, but we need ASCII */
+        if
+          (
+#if !HAVE_ICONV
+          (*hpart&0x80) || (*npart&0x80) ||
+#endif
+          ascii_caseless
+          ? ((*npart>='A' && *npart<='Z' ? *npart|0x20 : *npart) != (*hpart>='A' && *hpart<='Z' ? *hpart|0x20 : *hpart))
+          : *hpart!=*npart
+          )
+          {
+          if (may_advance)
+            /* string match after a star failed, advance and try again */
             {
-            ++h.character;
-            --h.length;
+            ++h;
+            goto match_part;
             }
+          else return 0;
           }
         else
           {
-          ++h.character;
-          --h.length;
-          }
+          ++npart;
+          ++hpart;
+          };
         }
-      else return 0;
-      break;
       }
-    case '\\':
+    /* at this point, a part was matched successfully */
+    if (may_advance && npart==nend && hpart<hend)
+      /* needle ends, but haystack does not: if there was a star before, advance and try again */
       {
-      ++n.character;
-      --n.length;
-      /* FALLTHROUGH */
-      }
-    default:
-      {
-      char nc,hc;
-
-      if (h.length==0) return 0;
-      nc=n.character[0];
-      hc=h.character[0];
-#if !HAVE_ICONV
-      if ((hc&0x80) || (nc&0x80)) return 0;
-#endif
-      /* tolower depends on the locale and only ASCII case must be insensitive */
-      if ((nc&0x80) || (hc&0x80)) { if (nc!=hc) return 0; }
-      else if ((nc>='A' && nc<='Z' ? nc|0x20 : nc) != (hc>='A' && hc<='Z' ? hc|0x20 : hc)) return 0;
-      ++h.character;
-      --h.length;
-      ++n.character;
-      --n.length;
+      ++h;
+      goto match_part;
       }
+    h=hpart;
+    n=npart;
+    may_advance=0;
     }
   }
-return (h.length==0);
+return (h==hend ? 1 : may_advance);
 }
 
 
@@ -644,7 +538,7 @@ if ((filter_test != FTEST_NONE && debug_selector != 0) ||
   switch (co)
     {
     case COMP_OCTET: debug_printf("i;octet"); break;
-    case COMP_ASCII_CASEMAP: debug_printf("i;ascii-casemap"); break;
+    case COMP_EN_ASCII_CASEMAP: debug_printf("en;ascii-casemap"); break;
     case COMP_ASCII_NUMERIC: debug_printf("i;ascii-numeric"); break;
     }
   debug_printf("\"):\n");
@@ -662,7 +556,7 @@ switch (mt)
         if (eq_octet(needle,haystack,0)) r=1;
         break;
         }
-      case COMP_ASCII_CASEMAP:
+      case COMP_EN_ASCII_CASEMAP:
         {
         if (eq_asciicase(needle,haystack,0)) r=1;
         break;
@@ -691,7 +585,7 @@ switch (mt)
         for (h=*haystack; h.length; ++h.character,--h.length) if (eq_octet(needle,&h,1)) { r=1; break; }
         break;
         }
-      case COMP_ASCII_CASEMAP:
+      case COMP_EN_ASCII_CASEMAP:
         {
         for (h=*haystack; h.length; ++h.character,--h.length) if (eq_asciicase(needle,&h,1)) { r=1; break; }
         break;
@@ -710,12 +604,20 @@ switch (mt)
       {
       case COMP_OCTET:
         {
-        if (eq_octetglob(needle,haystack)) r=1;
+        if ((r=eq_glob(needle,haystack,0))==-1)
+          {
+          filter->errmsg=CUS "syntactically invalid pattern";
+          return -1;
+          }
         break;
         }
-      case COMP_ASCII_CASEMAP:
+      case COMP_EN_ASCII_CASEMAP:
         {
-        if (eq_asciicaseglob(needle,haystack)) r=1;
+        if ((r=eq_glob(needle,haystack,1))==-1)
+          {
+          filter->errmsg=CUS "syntactically invalid pattern";
+          return -1;
+          }
         break;
         }
       default:
@@ -910,7 +812,7 @@ while (*r)
     *t++=*r++;
   }
 *t++='\0';
-value->character=rfc2047_decode(s,TRUE,US"utf-8",'\0',&value->length,&errmsg);
+value->character=rfc2047_decode(s,check_rfc2047_length,US"utf-8",'\0',&value->length,&errmsg);
 }
 
 
@@ -1468,7 +1370,12 @@ switch (parse_string(filter,&comparator_name))
       }
     else if (eq_asciicase(&comparator_name,&str_iascii_casemap,0))
       {
-      *c=COMP_ASCII_CASEMAP;
+      *c=COMP_EN_ASCII_CASEMAP;
+      match=1;
+      }
+    else if (eq_asciicase(&comparator_name,&str_enascii_casemap,0))
+      {
+      *c=COMP_EN_ASCII_CASEMAP;
       match=1;
       }
     else if (eq_asciicase(&comparator_name,&str_iascii_numeric,0))
@@ -1608,7 +1515,7 @@ if (parse_identifier(filter,CUS "address"))
   */
 
   enum AddressPart addressPart=ADDRPART_ALL;
-  enum Comparator comparator=COMP_ASCII_CASEMAP;
+  enum Comparator comparator=COMP_EN_ASCII_CASEMAP;
   enum MatchType matchType=MATCH_IS;
   struct String *hdr,*h,*key,*k;
   int m;
@@ -1711,10 +1618,8 @@ if (parse_identifier(filter,CUS "address"))
           case ADDRPART_LOCALPART: part=extracted_addr; part[domain-1]='\0'; break;
           case ADDRPART_DOMAIN: part=extracted_addr+domain; break;
 #ifdef SUBADDRESS
-          case ADDRPART_DETAIL:
-          part=NULL;
+          case ADDRPART_DETAIL: part=NULL; break;
 #endif
-          break;
           }
 
         *end_addr = saveend;
@@ -1820,7 +1725,7 @@ else if (parse_identifier(filter,CUS "header"))
                 <header-names: string-list> <key-list: string-list>
   */
 
-  enum Comparator comparator=COMP_ASCII_CASEMAP;
+  enum Comparator comparator=COMP_EN_ASCII_CASEMAP;
   enum MatchType matchType=MATCH_IS;
   struct String *hdr,*h,*key,*k;
   int m;
@@ -1941,9 +1846,12 @@ else if (parse_identifier(filter,CUS "envelope"))
                   <envelope-part: string-list> <key-list: string-list>
 
   envelope-part is case insensitive "from" or "to"
+#ifdef ENVELOPE_AUTH
+  envelope-part =/ "auth"
+#endif
   */
 
-  enum Comparator comparator=COMP_ASCII_CASEMAP;
+  enum Comparator comparator=COMP_EN_ASCII_CASEMAP;
   enum AddressPart addressPart=ADDRPART_ALL;
   enum MatchType matchType=MATCH_IS;
   struct String *env,*e,*key,*k;
@@ -2003,7 +1911,7 @@ else if (parse_identifier(filter,CUS "envelope"))
     return -1;
     }
   *cond=0;
-  for (e=env; e->character; ++e)
+  for (e=env; e->length!=-1 && !*cond; ++e)
     {
     const uschar *envelopeExpr=CUS 0;
     uschar *envelope=US 0;
@@ -2019,9 +1927,7 @@ else if (parse_identifier(filter,CUS "envelope"))
         case ADDRPART_LOCALPART: envelopeExpr=CUS "${local_part:$sender_address}"; break;
         case ADDRPART_DOMAIN: envelopeExpr=CUS "${domain:$sender_address}"; break;
 #ifdef SUBADDRESS
-        case ADDRPART_DETAIL:
-        envelopeExpr=CUS 0;
-        break;
+        case ADDRPART_DETAIL: envelopeExpr=CUS 0; break;
 #endif
         }
       }
@@ -2031,13 +1937,30 @@ else if (parse_identifier(filter,CUS "envelope"))
         {
         case ADDRPART_ALL: envelopeExpr=CUS "$local_part_prefix$local_part$local_part_suffix@$domain"; break;
 #ifdef SUBADDRESS
-        case ADDRPART_USER: envelopeExpr=CUS "$local_part_prefix$local_part"; break;
-        case ADDRPART_DETAIL: envelopeExpr=CUS "$local_part_suffix"; break;
+        case ADDRPART_USER: envelopeExpr=filter->useraddress; break;
+        case ADDRPART_DETAIL: envelopeExpr=filter->subaddress; break;
 #endif
         case ADDRPART_LOCALPART: envelopeExpr=CUS "$local_part_prefix$local_part$local_part_suffix"; break;
         case ADDRPART_DOMAIN: envelopeExpr=CUS "$domain"; break;
         }
       }
+#ifdef ENVELOPE_AUTH
+    else if (eq_asciicase(e,&str_auth,0))
+      {
+      switch (addressPart)
+        {
+        case ADDRPART_ALL: envelopeExpr=CUS "$authenticated_sender"; break;
+#ifdef SUBADDRESS
+        case ADDRPART_USER:
+#endif
+        case ADDRPART_LOCALPART: envelopeExpr=CUS "${local_part:$authenticated_sender}"; break;
+        case ADDRPART_DOMAIN: envelopeExpr=CUS "${domain:$authenticated_sender}"; break;
+#ifdef SUBADDRESS
+        case ADDRPART_DETAIL: envelopeExpr=CUS 0; break;
+#endif
+        }
+      }
+#endif
     else
       {
       filter->errmsg=CUS "invalid envelope string";
@@ -2056,7 +1979,7 @@ else if (parse_identifier(filter,CUS "envelope"))
 
         envelopeStr.character=envelope;
         envelopeStr.length=Ustrlen(envelope);
-        *cond=compare(filter,&envelopeStr,k,comparator,matchType);
+        *cond=compare(filter,k,&envelopeStr,comparator,matchType);
         if (*cond==-1) return -1;
         if (*cond) break;
         }
@@ -2173,6 +2096,11 @@ while (*filter->pc)
       filter->errmsg=CUS "missing test";
       return -1;
       }
+    if ((filter_test != FTEST_NONE && debug_selector != 0) ||
+        (debug_selector & D_filter) != 0)
+      {
+      if (exec) debug_printf("if %s\n",cond?"true":"false");
+      }
     m=parse_block(filter,exec ? cond : 0, generated);
     if (m==-1 || m==2) return m;
     if (m==0)
@@ -2194,6 +2122,11 @@ while (*filter->pc)
           filter->errmsg=CUS "missing test";
           return -1;
           }
+        if ((filter_test != FTEST_NONE && debug_selector != 0) ||
+            (debug_selector & D_filter) != 0)
+          {
+          if (exec) debug_printf("elsif %s\n",cond?"true":"false");
+          }
         m=parse_block(filter,exec && unsuccessful ? cond : 0, generated);
         if (m==-1 || m==2) return m;
         if (m==0)
@@ -2360,23 +2293,75 @@ while (*filter->pc)
       }
     if (parse_semicolon(filter)==-1) return -1;
     }
+#ifdef NOTIFY
+  else if (parse_identifier(filter,CUS "notify"))
+    {
+    /*
+    notify-command =  "notify" { notify-options } ";"
+    notify-options =  [":method" string]
+                      [":message" string]
+    */
+
+    int m;
+    struct String method;
+    struct String message;
+
+    if (!filter->require_notify)
+      {
+      filter->errmsg=CUS "missing previous require \"notify\";";
+      return -1;
+      }
+    method.character=(uschar*)0;
+    method.length=-1;
+    message.character=(uschar*)0;
+    message.length=-1;
+    for (;;)
+      {
+      if (parse_white(filter)==-1) return -1;
+      if (parse_identifier(filter,CUS ":method")==1)
+        {
+        if (parse_white(filter)==-1) return -1;
+        if ((m=parse_string(filter,&method))!=1)
+          {
+          if (m==0) filter->errmsg=CUS "method string expected";
+          return -1;
+          }
+        }
+      else if (parse_identifier(filter,CUS ":message")==1)
+        {
+        if (parse_white(filter)==-1) return -1;
+        if ((m=parse_string(filter,&message))!=1)
+          {
+          if (m==0) filter->errmsg=CUS "message string expected";
+          return -1;
+          }
+        }
+      else break;
+      }
+    if (parse_semicolon(filter)==-1) return -1;
+    }
+#endif
 #ifdef VACATION
   else if (parse_identifier(filter,CUS "vacation"))
     {
     /*
     vacation-command =  "vacation" { vacation-options } <reason: string> ";"
     vacation-options =  [":days" number]
-                        [":addresses" string-list]
                         [":subject" string]
+                        [":from" string]
+                        [":addresses" string-list]
                         [":mime"]
+                        [":handle" string]
     */
 
     int m;
     unsigned long days;
-    struct String *addresses;
     struct String subject;
+    struct String from;
+    struct String *addresses;
     int reason_is_mime;
     string_item *aliases;
+    struct String handle;
     struct String reason;
 
     if (!filter->require_vacation)
@@ -2394,11 +2379,15 @@ while (*filter->pc)
       filter->vacation_ran=1;
       }
     days=VACATION_MIN_DAYS>7 ? VACATION_MIN_DAYS : 7;
-    addresses=(struct String*)0;
     subject.character=(uschar*)0;
     subject.length=-1;
+    from.character=(uschar*)0;
+    from.length=-1;
+    addresses=(struct String*)0;
     aliases=NULL;
     reason_is_mime=0;
+    handle.character=(uschar*)0;
+    handle.length=-1;
     for (;;)
       {
       if (parse_white(filter)==-1) return -1;
@@ -2409,6 +2398,43 @@ while (*filter->pc)
         if (days<VACATION_MIN_DAYS) days=VACATION_MIN_DAYS;
         else if (days>VACATION_MAX_DAYS) days=VACATION_MAX_DAYS;
         }
+      else if (parse_identifier(filter,CUS ":subject")==1)
+        {
+        if (parse_white(filter)==-1) return -1;
+        if ((m=parse_string(filter,&subject))!=1)
+          {
+          if (m==0) filter->errmsg=CUS "subject string expected";
+          return -1;
+          }
+        }
+      else if (parse_identifier(filter,CUS ":from")==1)
+        {
+        int start, end, domain;
+        uschar *error,*ss;
+
+        if (parse_white(filter)==-1) return -1;
+        if ((m=parse_string(filter,&from))!=1)
+          {
+          if (m==0) filter->errmsg=CUS "from string expected";
+          return -1;
+          }
+        if (from.length>0)
+          {
+          ss = parse_extract_address(from.character, &error, &start, &end, &domain,
+            FALSE);
+          if (ss == NULL)
+            {
+            filter->errmsg=string_sprintf("malformed address \"%s\" in "
+              "Sieve filter: %s", from.character, error);
+            return -1;
+            }
+          }
+        else
+          {
+          filter->errmsg=CUS "empty :from address in Sieve filter";
+          return -1;
+          }
+        }
       else if (parse_identifier(filter,CUS ":addresses")==1)
         {
         struct String *a;
@@ -2431,17 +2457,17 @@ while (*filter->pc)
           aliases=new;
           }
         }
-      else if (parse_identifier(filter,CUS ":subject")==1)
+      else if (parse_identifier(filter,CUS ":mime")==1)
+        reason_is_mime=1;
+      else if (parse_identifier(filter,CUS ":handle")==1)
         {
         if (parse_white(filter)==-1) return -1;
-        if ((m=parse_string(filter,&subject))!=1)
+        if ((m=parse_string(filter,&from))!=1)
           {
-          if (m==0) filter->errmsg=CUS "subject string expected";
+          if (m==0) filter->errmsg=CUS "handle string expected";
           return -1;
           }
         }
-      else if (parse_identifier(filter,CUS ":mime")==1)
-        reason_is_mime=1;
       else break;
       }
     if (parse_white(filter)==-1) return -1;
@@ -2450,6 +2476,17 @@ while (*filter->pc)
       if (m==0) filter->errmsg=CUS "missing reason string";
       return -1;
       }
+    if (reason_is_mime)
+      {
+      uschar *s,*end;
+
+      for (s=reason.character,end=reason.character+reason.length; s<end && (*s&0x80)==0; ++s);
+      if (s<end)
+        {
+        filter->errmsg=CUS "MIME reason string contains 8bit text";
+        return -1;
+        }
+      }
     if (parse_semicolon(filter)==-1) return -1;
 
     if (exec)
@@ -2459,7 +2496,6 @@ while (*filter->pc)
       uschar *buffer;
       int buffer_capacity;
       struct String key;
-      struct String *a;
       md5 base;
       uschar digest[16];
       uschar hexdigest[33];
@@ -2468,112 +2504,123 @@ while (*filter->pc)
 
       if (filter_personal(aliases,TRUE))
         {
+        if (filter_test == FTEST_NONE)
+          {
+          /* ensure oncelog directory exists; failure will be detected later */
 
-        /* ensure oncelog directory exists; failure will be detected later */
-
-        (void)directory_make(NULL, filter->vacation_directory, 0700, FALSE);
-
+          (void)directory_make(NULL, filter->vacation_directory, 0700, FALSE);
+          }
         /* build oncelog filename */
 
         key.character=(uschar*)0;
         key.length=0;
         capacity=0;
-        if (subject.length!=-1) key.character=string_cat(key.character,&capacity,&key.length,subject.character,subject.length);
-        key.character=string_cat(key.character,&capacity,&key.length,reason_is_mime?US"1":US"0",1);
-        key.character=string_cat(key.character,&capacity,&key.length,reason.character,reason.length);
-        if (addresses!=(struct String*)0) for (a=addresses; a->length!=-1; ++a)
+        if (handle.length==-1)
           {
-          key.character=string_cat(key.character,&capacity,&key.length,US":",1);
-          key.character=string_cat(key.character,&capacity,&key.length,a->character,a->length);
+          if (subject.length!=-1) key.character=string_cat(key.character,&capacity,&key.length,subject.character,subject.length);
+          if (from.length!=-1) key.character=string_cat(key.character,&capacity,&key.length,from.character,from.length);
+          key.character=string_cat(key.character,&capacity,&key.length,reason_is_mime?US"1":US"0",1);
+          key.character=string_cat(key.character,&capacity,&key.length,reason.character,reason.length);
           }
+        else
+          key=handle;
         md5_start(&base);
         md5_end(&base, key.character, key.length, digest);
         for (i = 0; i < 16; i++) sprintf(CS (hexdigest+2*i), "%02X", digest[i]);
-        capacity=Ustrlen(filter->vacation_directory);
-        start=capacity;
-        once=string_cat(filter->vacation_directory,&capacity,&start,US"/",1);
-        once=string_cat(once,&capacity,&start,hexdigest,33);
-        once[start] = '\0';
-
-        /* process subject */
-
-        if (subject.length==-1)
+        if (filter_test != FTEST_NONE)
           {
-          expand_header(&subject,&str_subject);
-          while (subject.length>=4 && Ustrncmp(subject.character,"Re: ",4)==0)
-          {
-            subject.character+=4;
-            subject.length-=4;
-          }
-          capacity=6;
-          start=6;
-          subject.character=string_cat(US"Auto: ",&capacity,&start,subject.character,subject.length);
-          subject.length=start;
-          }
-
-        /* add address to list of generated addresses */
-
-        addr = deliver_make_addr(string_sprintf(">%.256s", sender_address), FALSE);
-        setflag(addr, af_pfr);
-        setflag(addr, af_ignore_error);
-        addr->next = *generated;
-        *generated = addr;
-        addr->reply = store_get(sizeof(reply_item));
-        memset(addr->reply,0,sizeof(reply_item)); /* XXX */
-        addr->reply->to = string_copy(sender_address);
-        addr->reply->from = expand_string(US"$local_part@$domain");
-        /* Allocation is larger than neccessary, but enough even for split MIME words */
-        buffer_capacity=16+4*subject.length;
-        buffer=store_get(buffer_capacity);
-        addr->reply->subject=parse_quote_2047(subject.character, subject.length, US"utf-8", buffer, buffer_capacity);
-        addr->reply->oncelog=once;
-        addr->reply->once_repeat=days*86400;
-
-        /* build body and MIME headers */
-
-        if (reason_is_mime)
-          {
-          uschar *mime_body,*reason_end;
-#ifdef RFC_EOL
-          static const uschar nlnl[]="\r\n\r\n";
-#else
-          static const uschar nlnl[]="\n\n";
-#endif
-
-          for
-            (
-            mime_body=reason.character,reason_end=reason.character+reason.length;
-            mime_body<(reason_end-sizeof(nlnl)-1) && memcmp(mime_body,nlnl,sizeof(nlnl)-1);
-            ++mime_body
-            );
-          capacity = 0;
-          start = 0;
-          addr->reply->headers = string_cat(NULL,&capacity,&start,reason.character,mime_body-reason.character);
-          addr->reply->headers[start] = '\0';
-          capacity = 0;
-          start = 0;
-          if (mime_body+(sizeof(nlnl)-1)<reason_end) mime_body+=sizeof(nlnl)-1;
-          else mime_body=reason_end-1;
-          addr->reply->text = string_cat(NULL,&capacity,&start,mime_body,reason_end-mime_body);
-          addr->reply->text[start] = '\0';
+          debug_printf("Sieve: mail was personal, vacation file basename: %s\n", hexdigest);
           }
         else
           {
-          struct String qp;
+          capacity=Ustrlen(filter->vacation_directory);
+          start=capacity;
+          once=string_cat(filter->vacation_directory,&capacity,&start,US"/",1);
+          once=string_cat(once,&capacity,&start,hexdigest,33);
+          once[start] = '\0';
 
-          capacity = 0;
-          start = reason.length;
-          addr->reply->headers = US"MIME-Version: 1.0\n"
-                                 "Content-Type: text/plain;\n"
-                                 "\tcharset=\"utf-8\"\n"
-                                 "Content-Transfer-Encoding: quoted-printable";
-          addr->reply->text = quoted_printable_encode(&reason,&qp)->character;
+          /* process subject */
+
+          if (subject.length==-1)
+            {
+            expand_header(&subject,&str_subject);
+            capacity=6;
+            start=6;
+            subject.character=string_cat(US"Auto: ",&capacity,&start,subject.character,subject.length);
+            subject.length=start;
+            }
+
+          /* add address to list of generated addresses */
+
+          addr = deliver_make_addr(string_sprintf(">%.256s", sender_address), FALSE);
+          setflag(addr, af_pfr);
+          setflag(addr, af_ignore_error);
+          addr->next = *generated;
+          *generated = addr;
+          addr->reply = store_get(sizeof(reply_item));
+          memset(addr->reply,0,sizeof(reply_item)); /* XXX */
+          addr->reply->to = string_copy(sender_address);
+          if (from.length==-1)
+            addr->reply->from = expand_string(US"$local_part@$domain");
+          else
+            addr->reply->from = from.character;
+          /* Allocation is larger than neccessary, but enough even for split MIME words */
+          buffer_capacity=16+4*subject.length;
+          buffer=store_get(buffer_capacity);
+          addr->reply->subject=parse_quote_2047(subject.character, subject.length, US"utf-8", buffer, buffer_capacity);
+          addr->reply->oncelog=once;
+          addr->reply->once_repeat=days*86400;
+
+          /* build body and MIME headers */
+
+          if (reason_is_mime)
+            {
+            uschar *mime_body,*reason_end;
+#ifdef RFC_EOL
+            static const uschar nlnl[]="\r\n\r\n";
+#else
+            static const uschar nlnl[]="\n\n";
+#endif
+
+            for
+              (
+              mime_body=reason.character,reason_end=reason.character+reason.length;
+              mime_body<(reason_end-sizeof(nlnl)-1) && memcmp(mime_body,nlnl,sizeof(nlnl)-1);
+              ++mime_body
+              );
+            capacity = 0;
+            start = 0;
+            addr->reply->headers = string_cat(NULL,&capacity,&start,reason.character,mime_body-reason.character);
+            addr->reply->headers[start] = '\0';
+            capacity = 0;
+            start = 0;
+            if (mime_body+(sizeof(nlnl)-1)<reason_end) mime_body+=sizeof(nlnl)-1;
+            else mime_body=reason_end-1;
+            addr->reply->text = string_cat(NULL,&capacity,&start,mime_body,reason_end-mime_body);
+            addr->reply->text[start] = '\0';
+            }
+          else
+            {
+            struct String qp;
+
+            capacity = 0;
+            start = reason.length;
+            addr->reply->headers = US"MIME-Version: 1.0\n"
+                                   "Content-Type: text/plain;\n"
+                                   "\tcharset=\"utf-8\"\n"
+                                   "Content-Transfer-Encoding: quoted-printable";
+            addr->reply->text = quoted_printable_encode(&reason,&qp)->character;
+            }
           }
         }
+        else if (filter_test != FTEST_NONE)
+          {
+          debug_printf("Sieve: mail was not personal, vacation would ignore it\n");
+          }
       }
     }
+    else break;
 #endif
-  else break;
   }
 return 1;
 }
@@ -2601,6 +2648,12 @@ filter->line=1;
 filter->keep=1;
 filter->require_envelope=0;
 filter->require_fileinto=0;
+#ifdef ENVELOPE_AUTH
+filter->require_envelope_auth=0;
+#endif
+#ifdef NOTIFY
+filter->require_notify=0;
+#endif
 #ifdef SUBADDRESS
 filter->require_subaddress=0;
 #endif
@@ -2613,7 +2666,7 @@ filter->require_iascii_numeric=0;
 
 if (parse_white(filter)==-1) return -1;
 
-if (exec && filter->vacation_directory != NULL)   /* 2nd test added by PH */
+if (exec && filter->vacation_directory != NULL && filter_test == FTEST_NONE)
   {
   DIR *oncelogdir;
   struct dirent *oncelog;
@@ -2664,15 +2717,21 @@ while (parse_identifier(filter,CUS "require"))
     }
   for (check=cap; check->character; ++check)
     {
-    if (eq_asciicase(check,&str_envelope,0)) filter->require_envelope=1;
-    else if (eq_asciicase(check,&str_fileinto,0)) filter->require_fileinto=1;
+    if (eq_octet(check,&str_envelope,0)) filter->require_envelope=1;
+    else if (eq_octet(check,&str_fileinto,0)) filter->require_fileinto=1;
+#ifdef ENVELOPE_AUTH
+    else if (eq_octet(check,&str_envelope_auth,0)) filter->require_envelope_auth=1;
+#endif
+#ifdef NOTIFY
+    else if (eq_octet(check,&str_notify,0)) filter->require_notify=1;
+#endif
 #ifdef SUBADDRESS
-    else if (eq_asciicase(check,&str_subaddress,0)) filter->require_subaddress=1;
+    else if (eq_octet(check,&str_subaddress,0)) filter->require_subaddress=1;
 #endif
 #ifdef VACATION
-    else if (eq_asciicase(check,&str_vacation,0))
+    else if (eq_octet(check,&str_vacation,0))
       {
-      if (filter->vacation_directory == NULL)
+      if (filter_test == FTEST_NONE && filter->vacation_directory == NULL)
         {
         filter->errmsg=CUS "vacation disabled";
         return -1;
@@ -2680,10 +2739,11 @@ while (parse_identifier(filter,CUS "require"))
       filter->require_vacation=1;
       }
 #endif
-    else if (eq_asciicase(check,&str_copy,0)) filter->require_copy=1;
-    else if (eq_asciicase(check,&str_comparator_ioctet,0)) ;
-    else if (eq_asciicase(check,&str_comparator_iascii_casemap,0)) ;
-    else if (eq_asciicase(check,&str_comparator_iascii_numeric,0)) filter->require_iascii_numeric=1;
+    else if (eq_octet(check,&str_copy,0)) filter->require_copy=1;
+    else if (eq_octet(check,&str_comparator_ioctet,0)) ;
+    else if (eq_octet(check,&str_comparator_iascii_casemap,0)) ;
+    else if (eq_octet(check,&str_comparator_enascii_casemap,0)) ;
+    else if (eq_octet(check,&str_comparator_iascii_numeric,0)) filter->require_iascii_numeric=1;
     else
       {
       filter->errmsg=CUS "unknown capability";
@@ -2712,6 +2772,8 @@ Arguments:
   options     controls whether various special things are allowed, and requests
               special actions (not currently used)
   sieve_vacation_directory  where to store vacation "once" files
+  useraddress string expression for :user part of address
+  subaddress  string expression for :subaddress part of address
   generated   where to hang newly-generated addresses
   error       where to pass back an error text
 
@@ -2725,7 +2787,7 @@ Returns:      FF_DELIVERED     success, a significant action was taken
 
 int
 sieve_interpret(uschar *filter, int options, uschar *vacation_directory,
-  address_item **generated, uschar **error)
+  uschar *useraddress, uschar *subaddress, address_item **generated, uschar **error)
 {
 struct Sieve sieve;
 int r;
@@ -2751,6 +2813,9 @@ else
     }
   }
 
+sieve.useraddress = useraddress == NULL ? CUS "$local_part_prefix$local_part$local_part_suffix" : useraddress;
+sieve.subaddress = subaddress;
+
 #ifdef COMPILE_SYNTAX_CHECKER
 if (parse_start(&sieve,0,generated)==1)
 #else
@@ -2760,12 +2825,12 @@ if (parse_start(&sieve,1,generated)==1)
   if (sieve.keep)
     {
     add_addr(generated,US"inbox",1,0,0,0);
-    msg = string_sprintf("Keep");
+    msg = string_sprintf("Implicit keep");
     r = FF_DELIVERED;
     }
-    else
+  else
     {
-    msg = string_sprintf("No keep");
+    msg = string_sprintf("No implicit keep");
     r = FF_DELIVERED;
     }
   }
